use core;
use core::fmt;
use serde::{de, ser};

// Result type
pub type Result<T> = core::result::Result<T, Error>;

// Error type
#[derive(Clone, Debug, PartialEq)]
pub struct Error(pub &'static str);

impl fmt::Display for Error {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        formatter.write_str(self.0)
    }
}

impl ser::Error for Error {
    fn custom<T: fmt::Display>(_msg: T) -> Self {
        Error("serialization error")
    }
}

impl de::Error for Error {
    fn custom<T: fmt::Display>(_msg: T) -> Self {
        Error("deserialization error")
    }
}

impl From<&'static str> for Error {
    fn from(err: &'static str) -> Error {
        Error(err)
    }
}
