#![no_std]

extern crate byteorder;
extern crate num_traits;

#[macro_use]
extern crate serde;

// The serde_derive crate provides the macros for #[derive(Serialize)] and
// #[derive(Deserialize)]. You won't need these for implementing a data format
// but your unit tests will probably use them - hence #[cfg(test)].
#[cfg(test)]
#[macro_use]
extern crate serde_derive;
extern crate arrayvec;

mod error;
mod ser;
mod de;
pub mod rmp;

pub use error::{Error, Result};
pub use ser::{write_named, Serializer};
pub use de::{from_slice, Deserializer};

use arrayvec::{Array, ArrayVec};

impl<A: Array<Item=u8>> rmp::io::Write for ArrayVec<A> {
    fn write_all(&mut self, buf: &[u8]) -> Result<()> {
        unsafe {
            let old_len = self.len();
            let new_len = old_len + buf.len();

            if new_len > A::capacity() {
                return Err(Error("buffer is full"));
            }
            {
                let dst = self.get_unchecked_mut(old_len .. new_len);
                dst.copy_from_slice(buf);
            }
            self.set_len(new_len);
        }
        Ok(())
    }
}
